/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Gmail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Doron Rosenberg.
 * Portions created by the Initial Developer are Copyright (C) 2004 - 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const kGMSERVICE_CONTRACTID = "@mozilla.org/GMailNotifier;1"
const kGMSERVICE_CID = Components.ID("02509a0c-249e-4396-8f72-fdf5af33d7b4");
const nsIGMNotifierService = Components.interfaces.nsIGMNotifierService
const nsINotifierProgressListener = Components.interfaces.nsIGMNotifierProgressListener;
const nsISupports = Components.interfaces.nsISupports;

const kIOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
const nsIIOService = Components.interfaces.nsIIOService;

const nsICookieManager = Components.interfaces.nsICookieManager;
const kCOOKIESERVICE_CONTRACTID = "@mozilla.org/cookieService;1";

const kPSWDMANAGER_CONTRACTID = "@mozilla.org/passwordmanager;1";
const nsIPasswordManagerInternal = Components.interfaces.nsIPasswordManagerInternal;

const kTIMER_CONTRACTID = "@mozilla.org/timer;1";
const nsITimer = Components.interfaces.nsITimer;

function nsGMNotifierService() {
  // log string
  this.logString = "";
  this.listeners = new Array();
  this.username = null;
  this.password = null;
  this.notificationListenerID = null;

  this.resetState = false;
  this.connectionPhase = null;

  this.prefBranch = null;

  this.folders = new Array();

  this.unreadEmails = null;
  this.newEmails = 0;
  this.space_used_mb = null;
  this.space_used_percent = null;
  this.total_mb = null;
  this.inboxUnread = 0;
  this.inboxNew = 0;

  this.updateTimer = null;
  this.timeOut = 600000; // default to 10 mins

  var myThis = this;
  this.PrefChangeObserver = {
    observe: function(aSubject, aTopic, aData)
    {
      myThis.prefChanged(aData);
    }
  };

  this.addObserver("gm-notifier", this.PrefChangeObserver);
}

/********** Scriptable interfaces ****************/

/**
 * Initiates login
 *
 * @param aUsername
 * @param aPassword
 * @param aListenerID
 */
nsGMNotifierService.prototype.initLogin = function(aUsername, aPassword, aListenerID) {
  gGMNotifierService.logItem("initLogin: aListenerID is " + aListenerID);
  gGMNotifierService.username = aUsername;
  gGMNotifierService.password = aPassword;

  gGMNotifierService.notificationListenerID = aListenerID;

  gGMNotifierService.startConnection();
}

/**
 * Initiates a new mail check
 *
 */
nsGMNotifierService.prototype.checkNow = function() {
  gGMNotifierService.startConnection();
}

/**
 * Logs the user out
 *
 */
nsGMNotifierService.prototype.logout = function() {
  this.username = null;
  this.password = null;

  this.unreadEmails = null;
  this.newEmails = 0;
  this.space_used_mb = null;
  this.space_used_percent = null;
  this.total_mb = null;

  // clear the timer
  if (this.updateTimer)
    this.updateTimer.cancel();
  this.updateTimer = null;

  this.loggedIn = false;

  // clear folders
  this.folders = new Array();
  this.inboxUnread = 0;

  // clear the login cookie
  try{
    var ios = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
    var cookMan = Components.classes[kCOOKIESERVICE_CONTRACTID].getService(nsICookieManager);
    cookMan.remove("mail.google.com", "GV", "/", false);
  } catch(e) {}

  // tell all listeners that we logged out
  this.pushStateChange(nsINotifierProgressListener.LOGOUT);
}

/**
 * Adds an NotifierListener to this service
 *
 * @param aListener
 */
nsGMNotifierService.prototype.addListener = function(aListener) {
  // should we autologin?
  // make sure to only check for autologin the first time a listener has been
  // added, ie first browser window calls us.
  if ((this.listeners.length == 0) && this.getPrefBranch() &&
      this.getPrefBranch().getBoolPref("gm-notifier.autologin.enabled")) {
    // autologin
    var defaultUser = this.getPrefBranch().getCharPref("gm-notifier.users.default");
    if (defaultUser) {
      this.username = defaultUser;
      this.password = this.getLoginDetails(this.username);
      this.startConnection();
    }
  }

  this.listeners.push(aListener);

  // for every new listener added, push login state
  if (this.loggedIn)
    aListener.onStateChange(nsINotifierProgressListener.LOGIN_SUCCESS);
  else
    aListener.onStateChange(nsINotifierProgressListener.LOGOUT);

  return this.listeners.length;
}

nsGMNotifierService.prototype.removeListener = function(aListener) {
  var found = false;
  var run = 0;

  while (!found || (run < this.listeners.length)) {
    if (this.listeners[run] == aListener) {
      this.listeners[run] = null;
      found = true;
    }
    run++;
  }
}

nsGMNotifierService.prototype.getResetState = function() {
  return this.resetState;
}

nsGMNotifierService.prototype.setResetState = function(aResetState) {
  this.resetState = aResetState;
  gGMNotifierService.pushStateChange(nsINotifierProgressListener.LOGIN_SUCCESS);
}

nsGMNotifierService.prototype.getInboxUnread = function() {
  return this.inboxUnread;
}

nsGMNotifierService.prototype.getUnreadCount = function() {
  return this.unreadEmails;
}

nsGMNotifierService.prototype.getDisplayCount = function() {
  if (this.resetState) {
    return 0;
  } else {
    if (this.prefBranch.getBoolPref("gm-notifier.ui.counter.showInbox"))
     return this.getInboxUnread();
    else
      return this.unreadEmails;
  }
}

nsGMNotifierService.prototype.getNewCount = function() {
  var value;

  if (this.resetState) {
    value = 0;
  } else {
    if (this.prefBranch.getBoolPref("gm-notifier.ui.counter.showInbox"))
      value = this.inboxNew;
    else
      value = this.newEmails;
  }

  return value;
}

nsGMNotifierService.prototype.getUsedMB = function() {
  return this.space_used_mb;
}

nsGMNotifierService.prototype.getSpaceUsed = function() {
  return this.space_used_percent;
}

nsGMNotifierService.prototype.getTotalSpace = function() {
  return this.total_mb;
}

nsGMNotifierService.prototype.getUserName = function() {
  return this.username;
}

nsGMNotifierService.prototype.getFolderCount = function() {
  return this.folders.length;
}

nsGMNotifierService.prototype.getFolderItem = function (aPosition, sizeObj) {
  sizeObj.value = 2;
  return [this.folders[aPosition].name, this.folders[aPosition].unreadMail];
}

nsGMNotifierService.prototype.getUserName = function() {
  return this.username;
}

nsGMNotifierService.prototype.setTimeout = function(aMinutes) {
  // convert to ms
  this.timeOut = (aMinutes * 60000);

  // set timer only if we are logged in
  if (this.loggedIn)
    this.setTimer();
}

/**
 * Adds an item to the log
 *
 */
nsGMNotifierService.prototype.logItem = function(aLog) {
  this.logString += aLog + "|||";
  // dump("\n::::: " + aLog + "\n");
}

nsGMNotifierService.prototype.getLog = function() {
  return this.logString;
}

/*************** Hidden Methods ************/


/**
 * talkToServer and callback are the Gmail specific parts.
 */ 
nsGMNotifierService.prototype.startConnection = function () {

  gGMNotifierService.logItem("  StartConnection: Init");

  if (!this.username || !this.password) {
    return;
  }

  gGMNotifierService.logItem("  -- Username/Password found");

  this.resetState = false;

  this.connectionPhase = 0;
  this.pushStateChange(nsINotifierProgressListener.LOGIN_INITIATED);

  var data = "?service=mail&Email=" + encodeURIComponent(this.username)
             + "&Passwd=" + encodeURIComponent(this.password) +
             "&&rm=false&null=Sign%20in&continue=http://mail.google.com/mail?ui=html&amp;zy=l";

  this.talkToServer("https://www.google.com/accounts/ServiceLoginAuth", data,
                    null, "https://www.google.com/", this.callback);
}

/*
   Phase              Description
   0                  Before anything has happend
   1                  First connection
   ...
*/
nsGMNotifierService.prototype.talkToServer = function (aURL, aPostData, aCookieData, aReferrer, aCallbackFunc){

  gGMNotifierService.logItem("  talkeToServer Called");

  // the IO service
  var ioService = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
      
  // create an nsIURI
  var uri = ioService.newURI(aURL, null, null);

  //nsIInputStream
  var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
    .createInstance(Components.interfaces.nsIStringInputStream);

  if (aPostData)
    uploadStream.setData(aPostData, aPostData.length);

  // get a channel for that nsIURI
  var channel = ioService.newChannelFromURI(uri);

  // get a httpchannel and make it a post
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);

  // set a referrer
  if (aReferrer) {
    var referrerUri = ioService.newURI(aReferrer, null, null);
    httpChannel.referrer = referrerUri;
  }

  if (aPostData) {
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);

    // order important - setUploadStream resets to get/put
    httpChannel.requestMethod = "POST";
  }

  if (aCookieData){
    // httpChannel.setRequestHeader("Cookie", aCookieData, false);
    for (var run = 0; run < aCookieData.length; run++){
      httpChannel.setRequestHeader("Cookie", aCookieData[run], true);
    }
  }

  var observer = new this.observer(aCallbackFunc);
  channel.asyncOpen(observer, null);
}

nsGMNotifierService.prototype.callback = function (aData, aRequest) {
  gGMNotifierService.connectionPhase++;

  gGMNotifierService.logItem("    callback called, phase" + gGMNotifierService.connectionPhase);

dump("\n ------------------ \n" + gGMNotifierService.connectionPhase + "\n" +aData + "\n");



  switch (gGMNotifierService.connectionPhase) {
    case 1:
      var cookieData;

      // if no cookies sent, an exception is thrown
      /* try{
          var httpChannel = aRequest.QueryInterface(Components.interfaces.nsIHttpChannel)
          cookieData = httpChannel.getRequestHeader("cookie");
        } catch(e){} */
      var val = aData.indexOf("location.replace");

      if (val < 0) {
        // XXX - check if its an image request

        // failed to login
        gGMNotifierService.pushStateChange(nsINotifierProgressListener.LOGIN_FAILED);
        return;
      }

      //var url =  val[0].substr(2, (val[0].length-3) );
      /* var val = aData.match(/var cookieVal= "([^"]+)";/); */

      gGMNotifierService.talkToServer("http://mail.google.com/mail?ui",
             null, null, "https://www.google.com/", gGMNotifierService.callback);
      break;

    case 2:
      gGMNotifierService.talkToServer("https://mail.google.com/mail/?search=query&q=is:unread&view=tl&start=20",
                                       null, null, "https://www.google.com/",
                                       gGMNotifierService.callback);
      break;

    case 3:
      gGMNotifierService.logItem("  connecting to gmail complete.");

      var val = aData.match(/\["ds",.*\]/);

      if (!val) {
        // failed to login
        gGMNotifierService.logItem("  Login failed, final page not the correct one.");
        gGMNotifierService.pushStateChange(nsINotifierProgressListener.LOGIN_FAILED);
        return;
      }

      // get the unread count
      var ts = aData.match(/\["ts",.*\]/);
      var myArray = eval(ts[0]);
        gGMNotifierService.logItem("  item 1: " + myArray[0]);
        gGMNotifierService.logItem("  item 2: " + myArray[1]);
        gGMNotifierService.logItem("  item 3: " + myArray[2]);
        gGMNotifierService.logItem("  item 4: " + myArray[3]);
        gGMNotifierService.logItem("  item 5: " + myArray[4]);

      var totalUnread = myArray[3];

      // handle inbox
      myArray = eval(val[0]);
      gGMNotifierService.inboxNew = myArray[1] - gGMNotifierService.inboxUnread;
      gGMNotifierService.inboxUnread = myArray[1];
      gGMNotifierService.folders[0] = gGMNotifierService.createFolder("Inbox", myArray[1]);

      if (gGMNotifierService.inboxNew < 0)
        gGMNotifierService.inboxNew = 0;


      // handle labels
      var labelsVal = aData.match(/\["ct",.*\n(.*]\n)+\]/gm);
      if (labelsVal) {
        var labelsArray = eval(labelsVal[0])[1];
        for (var run = 0; run < labelsArray.length; run++) {
          gGMNotifierService.folders[run + 1] = gGMNotifierService.createFolder(labelsArray[run][0], labelsArray[run][1]);
          gGMNotifierService.logItem("  Folder found (name: " + gGMNotifierService.folders[run + 1].name + " and unread emails: " +  gGMNotifierService.folders[run + 1].unreadMail + ")");
				}
			}

      // new/unread mail check
      gGMNotifierService.logItem("GMail new mail check complete:");
      gGMNotifierService.logItem("  ::myArray[1] is " + myArray[1] + " unread is " + gGMNotifierService.unreadEmails);

      var showInboxOnly = gGMNotifierService.prefBranch.getBoolPref("gm-notifier.ui.counter.showInbox");
      var hasNewMail = (totalUnread > gGMNotifierService.unreadEmails);

      if (!hasNewMail && showInboxOnly) {
        hasNewMail = (gGMNotifierService.inboxUnread > 0);
      }

      if ((gGMNotifierService.unreadEmails != null) && hasNewMail) {
        gGMNotifierService.logItem("  New Unread Mail Found!");
        // new email
        gGMNotifierService.newEmails = totalUnread - gGMNotifierService.unreadEmails;

        if (showInboxOnly && (gGMNotifierService.inboxUnread == 0)) {
          // show inbox only, but new mail wasn't in inbox
          gGMNotifierService.pushStateChange(nsINotifierProgressListener.N0_NEW_MAIL);
        } else {
          gGMNotifierService.pushStateChange(nsINotifierProgressListener.NEW_MAIL);
          gGMNotifierService.newMailNotification(gGMNotifierService.getNewCount());
        }
      } else if (gGMNotifierService.unreadEmails == null) {
        // first time
        if (totalUnread > 0) {
          gGMNotifierService.logItem("  New Mail Found!");
          gGMNotifierService.newEmails = totalUnread;
          gGMNotifierService.unreadEmails = totalUnread;
          gGMNotifierService.pushStateChange(nsINotifierProgressListener.NEW_MAIL);
          gGMNotifierService.logItem("    -- amount of new mail is " + gGMNotifierService.newEmails);
          gGMNotifierService.logItem("      val[0] is " + val[0]);
          gGMNotifierService.logItem("      myArray[1] is " + myArray[1]);
          gGMNotifierService.newMailNotification(gGMNotifierService.getDisplayCount());
        } else {
          gGMNotifierService.newEmails = 0;
          gGMNotifierService.unreadEmails = 0;
        }
      } else {
        gGMNotifierService.newEmails = 0;
        gGMNotifierService.logItem("  No New Unread Mail Found!");
        gGMNotifierService.pushStateChange(nsINotifierProgressListener.NO_NEW_MAIL);
      }

      gGMNotifierService.unreadEmails = totalUnread;

      val = aData.match(/\["qu",.*\]/);
      myArray = eval(val[0]);

      gGMNotifierService.space_used_mb = myArray[1];
      gGMNotifierService.total_mb = myArray[2];
      gGMNotifierService.space_used_percent = myArray[3];

      gGMNotifierService.loggedIn = true;

      gGMNotifierService.pushStateChange(nsINotifierProgressListener.LOGIN_SUCCESS);
      //gGMNotifierService.setTimer();
      break;
  }
}

// timing stuff
nsGMNotifierService.prototype.setTimer = function() {
  if (this.updateTimer) {
    this.updateTimer.cancel();
  } else {
    this.updateTimer = Components.classes[kTIMER_CONTRACTID].createInstance(nsITimer);
  }

  gGMNotifierService.logItem("Starting at : " + new Date());
  try{
    gGMNotifierService.updateTimer.initWithCallback(gGMNotifierService, gGMNotifierService.timeOut, nsITimer.TYPE_ONE_SHOT);
  } catch (e) {
    gGMNotifierService.logItem(e);
  }
}

// nsITimer
nsGMNotifierService.prototype.notify = function(aTimer) {
  gGMNotifierService.logItem("  *************** notify called");
  gGMNotifierService.logItem("End: " + new Date());
  gGMNotifierService.startConnection();
}

nsGMNotifierService.prototype.pushStateChange = function(aState) {
  for (var run = 0; run < this.listeners.length; run++)
    if (this.listeners[run]) // can be null if a listener was removed
      this.listeners[run].onStateChange(aState);
}

nsGMNotifierService.prototype.observer = function(aCallbackFunc) {
  return ({
    data : "",

    onStartRequest : function (aRequest, aContext) {
      this.data = "";
    },

    onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
      var scriptableInputStream = 
        Components.classes["@mozilla.org/scriptableinputstream;1"]
          .createInstance(Components.interfaces.nsIScriptableInputStream);
      scriptableInputStream.init(aStream);

      this.data += scriptableInputStream.read(aLength);
    },

    onStopRequest : function (aRequest, aContext, aStatus) {
      aCallbackFunc(this.data, aRequest);
    },

    QueryInterface : function(aIID) {
      if (aIID.equals(Components.interfaces.nsIStreamListener) ||
          aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
          aIID.equals(Components.interfaces.nsIAlertListener) ||
          aIID.equals(nsISupports))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  }
  );
}

/*
  New mail notification
 */
nsGMNotifierService.prototype.newMailNotification = function(aNewNum) {
  if (aNewNum == 0)
    return;

  this.logItem("New Mail Notification Init:");

  var isNotificationEnabled = this.getPrefBranch().getBoolPref("gm-notifier.ui.notification.enabled");
  this.logItem("  Is System Notification Enabled by user: " + isNotificationEnabled);

  if (isNotificationEnabled){
    var msg = this.getFormattedString("NotificationMsg", [aNewNum]);

    try {
      var alertService = Components.classes["@mozilla.org/alerts-service;1"]
                                   .getService(Components.interfaces.nsIAlertsService);
      if (alertService) {
        alertService.showAlertNotification("chrome://gm-notifier/content/gm-logo.png",
          this.getString("NotificationMsgTitle"), msg, true, "", this);
        this.logItem("  alertsService success.");
      } else {
       this.logItem("  alertsService failure: could not getService nsIAlertsService");
      }
    } catch(e) {
       this.logItem("  alertsService failure: " + e);
    }
  }

  // sound notifications
  var isSoundNotificationsEnabled = this.getPrefBranch().getBoolPref("gm-notifier.ui.soundnotification.enabled");
  this.logItem("  Is Sound Notification Enabled by user: " + isSoundNotificationsEnabled);

  if (isSoundNotificationsEnabled) {  
    var soundUrl = this.getPrefBranch().getCharPref("gm-notifier.ui.soundnotification.uri");

    try {
      var sound = Components.classes["@mozilla.org/sound;1"]
                            .createInstance(Components.interfaces.nsISound);
      if (soundUrl.indexOf("file://") == -1) {
        sound.playSystemSound(soundUrl);
      } else {
        var ioService = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
        var url = ioService.newURI(soundURL, null, null);
        sound.play(url)
      }
    } catch (e) {
      this.logItem("  Sound playing failed with: "+ e);
    }
  }
}

// nsIObserver for alert service
nsGMNotifierService.prototype.observe = function (aSubject, aTopic, aData){
  switch (aTopic) {
    case "alertfinished":
      break;
                   
    case "alertclickcallback":
      this.showNotification();
      break;
  }
}

// nsIAlertsService - nsIAlertListener was deprecated and replaced with nsIObserver
//                    This is to maintain 1.7 compatability.
nsGMNotifierService.prototype.onAlertFinished = function (aCookie){
}

nsGMNotifierService.prototype.onAlertClickCallback = function (aCookie){
  this.showNotification();
}

nsGMNotifierService.prototype.showNotification = function (){
  // when the alert callback happens, we want to notify the listener that
  // initiated the server connection that the alert was clicked.
  // Simply pushing a state change to all listeners will result in the
  // infinite window open loop!
  
  this.logItem("Alert Clickback called:");
  this.logItem("  Notification Window is: (" + gGMNotifierService.notificationListenerID + ").");

  var run = 0;
  if (gGMNotifierService.notificationListenerID) {
    var done = false;
    while (!done && (run < this.listeners.length)) {
      this.logItem("  Item " + run +  " has id (" + this.listeners[run].getID() + ")");
      if (this.listeners[run] && (this.listeners[run].getID() == this.notificationListenerID))
        done = true;
      else 
        run++;
    }
  } else {
    // fallback
    while ((this.listeners[run] == null) && (run < this.listeners.length))
      run++;

    this.logItem("  No notification window found, call listener #" + run);
  }

  if (this.listeners[run])
    this.listeners[run].onStateChange(nsINotifierProgressListener.LOAD_GMAIL);
}

nsGMNotifierService.prototype.createFolder = function(aName, aUnreadMail) {
  return {
    name: aName,
    unreadMail: aUnreadMail
  }
}

/* Pref code*/
nsGMNotifierService.prototype.getPrefBranch = function(){
  if (!this.prefBranch){ 
    this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'];
    this.prefBranch = this.prefBranch.getService();
    this.prefBranch = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch);
  }
  
  return this.prefBranch;
}

nsGMNotifierService.prototype.prefChanged = function(aPrefName) {
  switch(aPrefName){
    case "gm-notifier.update.interval":
      this.setTimeout(this.prefBranch.getIntPref("gm-notifier.update.interval"));
      break;
  }
}

nsGMNotifierService.prototype.addObserver = function(aDomain, aFunction){
  var myPrefs = this.getPrefBranch();
  var prefBranchInternal = myPrefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);

  if (prefBranchInternal)
    prefBranchInternal.addObserver(aDomain, aFunction, false);
}


/* Bundle code */
nsGMNotifierService.prototype.getStringBundle = function() {
  if (!this.stringBundle) {
    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
                             .createInstance(Components.interfaces.nsIStringBundleService);

    this.stringBundle = strBundleService.createBundle("chrome://gm-notifier/locale/gm-notifier.properties");
  }

  return this.stringBundle;
}

nsGMNotifierService.prototype.getString = function(aName) {
  return this.getStringBundle().GetStringFromName(aName);
}

nsGMNotifierService.prototype.getFormattedString = function(aName, aStrArray) {
  return this.getStringBundle().formatStringFromName(aName, aStrArray, aStrArray.length);
}

nsGMNotifierService.prototype.getLoginDetails = function(aUsername) {
  var url = "chrome://gm-notifier/";

  var passwordManager = Components.classes[kPSWDMANAGER_CONTRACTID]
                        .createInstance(nsIPasswordManagerInternal);
  var host = {value:""};
  var user =  {value:""};
  var password = {value:""};

  try {
    passwordManager.findPasswordEntry(url, aUsername, "", host, user, password);
  } catch(e){ }

  return password.value;
}

/**
 * JS XPCOM component registration goop:
 *
 * We set ourselves up to observe the xpcom-startup category.  This provides
 * us with a starting point.
 */

nsGMNotifierService.prototype.QueryInterface = function(iid) {
  if (!iid.equals(nsIGMNotifierService) &&
      !iid.equals(Components.interfaces.nsIAlertListener) &&
      !iid.equals(Components.interfaces.nsIObserver) &&
      !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_NO_INTERFACE;
  return this;
}

var gGMNotifierService = new nsGMNotifierService();

var nsGMNotifierServiceModule = new Object();

nsGMNotifierServiceModule.registerSelf = function (compMgr, fileSpec, location, type) {
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(kGMSERVICE_CID,
                                  "nsGMNotifierService",
                                  kGMSERVICE_CONTRACTID,
                                  fileSpec,
                                  location,
                                  type);
}

nsGMNotifierServiceModule.getClassObject = function (compMgr, cid, iid) {
  if (!cid.equals(kGMSERVICE_CID))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  if (!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

  return nsGMNotifierServiceFactory;
}

nsGMNotifierServiceModule.canUnload = function (compMgr) {
  // cleanup
  gGMNotifierService.listeners = null;

  if (gGMNotifierService.updateTimer)
    gGMNotifierService.updateTimer.cancel();
  gGMNotifierService.updateTimer = null;

  return true;
}

var nsGMNotifierServiceFactory = new Object();

nsGMNotifierServiceFactory.createInstance = function (outer, iid) {
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(nsIGMNotifierService) &&
      !iid.equals(Components.interfaces.nsIAlertListener) &&
      !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  return gGMNotifierService;
}

function NSGetModule(compMgr, fileSpec) {
  return nsGMNotifierServiceModule;
}

